﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Linq.Expressions;
using System.Text;

namespace XamNumericEditor_Ex
{
    public abstract class ObservableObject : INotifyPropertyChanged
    {
        protected bool IsValidating = false;

        public Dictionary<string, string> ErrorsFound = new Dictionary<string, string>();

        public virtual bool IsValid()
        {
            return true;
        }

        public virtual bool IsValid(IEnumerable<ObservableObject> items)
        {
            return true;
        }

        public event PropertyChangedEventHandler PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        protected virtual void OnPropertyChanged(Expression<Func<object>> expression)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (handler != null)
            {
                handler(this, new PropertyChangedEventArgs(expression.Name));
            }
        }

        protected bool SetField<T>(ref T field, T value, Expression<Func<object>> propertyExpression)
        {
            if (EqualityComparer<T>.Default.Equals(field, value))
            {
                return false;
            }
            field = value;
            OnPropertyChanged(propertyExpression);
            return true;
        }

        public virtual ObservableObject Clone()
        {
            return null;
        }
    }
}
